@@ -30,9 +30,8 @@ import warnings
 from collections.abc import Callable, Iterable, Sequence
 from contextlib import redirect_stderr, redirect_stdout
 from functools import lru_cache
-from pathlib import Path
 
-from astroid.const import IS_JYTHON, IS_PYPY, PY310_PLUS
+from astroid.const import IS_JYTHON, PY310_PLUS
 from astroid.interpreter._import import spec, util
 
 if PY310_PLUS:
@@ -45,9 +44,11 @@ logger = logging.getLogger(__name__)
 
 if sys.platform.startswith("win"):
     PY_SOURCE_EXTS = ("py", "pyw", "pyi")
+    PY_SOURCE_EXTS_STUBS_FIRST = ("pyi", "pyw", "py")
     PY_COMPILED_EXTS = ("dll", "pyd")
 else:
     PY_SOURCE_EXTS = ("py", "pyi")
+    PY_SOURCE_EXTS_STUBS_FIRST = ("pyi", "py")
     PY_COMPILED_EXTS = ("so",)
 
 
@@ -72,20 +73,6 @@ if os.name == "nt":
         except AttributeError:
             pass
 
-if IS_PYPY and sys.version_info < (3, 8):
-    # PyPy stores the stdlib in two places: sys.prefix/lib_pypy and sys.prefix/lib-python/3
-    # sysconfig.get_path on PyPy returns the first, but without an underscore so we patch this manually.
-    # Beginning with 3.8 the stdlib is only stored in: sys.prefix/pypy{py_version_short}
-    STD_LIB_DIRS.add(str(Path(sysconfig.get_path("stdlib")).parent / "lib_pypy"))
-    STD_LIB_DIRS.add(str(Path(sysconfig.get_path("stdlib")).parent / "lib-python/3"))
-
-    # TODO: This is a fix for a workaround in virtualenv. At some point we should revisit
-    # whether this is still necessary. See https://github.com/pylint-dev/astroid/pull/1324.
-    STD_LIB_DIRS.add(str(Path(sysconfig.get_path("platstdlib")).parent / "lib_pypy"))
-    STD_LIB_DIRS.add(
-        str(Path(sysconfig.get_path("platstdlib")).parent / "lib-python/3")
-    )
-
 if os.name == "posix":
     # Need the real prefix if we're in a virtualenv, otherwise
     # the usual one will do.
@@ -188,9 +175,10 @@ def load_module_from_name(dotted_name: str) -> types.ModuleType:
 
     # Capture and log anything emitted during import to avoid
     # contaminating JSON reports in pylint
-    with redirect_stderr(io.StringIO()) as stderr, redirect_stdout(
-        io.StringIO()
-    ) as stdout:
+    with (
+        redirect_stderr(io.StringIO()) as stderr,
+        redirect_stdout(io.StringIO()) as stdout,
+    ):
         module = importlib.import_module(dotted_name)
 
     stderr_value = stderr.getvalue()
@@ -484,7 +472,9 @@ def get_module_files(
     return files
 
 
-def get_source_file(filename: str, include_no_ext: bool = False) -> str:
+def get_source_file(
+    filename: str, include_no_ext: bool = False, prefer_stubs: bool = False
+) -> str:
     """Given a python module's file name return the matching source file
     name (the filename will be returned identically if it's already an
     absolute path to a python source file).
@@ -497,9 +487,10 @@ def get_source_file(filename: str, include_no_ext: bool = False) -> str:
     """
     filename = os.path.abspath(_path_from_filename(filename))
     base, orig_ext = os.path.splitext(filename)
-    if orig_ext == ".pyi" and os.path.exists(f"{base}{orig_ext}"):
-        return f"{base}{orig_ext}"
-    for ext in PY_SOURCE_EXTS:
+    orig_ext = orig_ext.lstrip(".")
+    if orig_ext not in PY_SOURCE_EXTS and os.path.exists(f"{base}.{orig_ext}"):
+        return f"{base}.{orig_ext}"
+    for ext in PY_SOURCE_EXTS_STUBS_FIRST if prefer_stubs else PY_SOURCE_EXTS:
         source_path = f"{base}.{ext}"
         if os.path.exists(source_path):
             return source_path
@@ -666,6 +657,7 @@ def _is_python_file(filename: str) -> bool:
     return filename.endswith((".py", ".pyi", ".so", ".pyd", ".pyw"))
 
 
+@lru_cache(maxsize=1024)
 def _has_init(directory: str) -> str | None:
     """If the given directory has a valid __init__ file, return its path,
     else return None.
