@@ -1,14 +1,15 @@
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
-# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
+# For details: https://github.com/pylint-dev/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/astroid/blob/main/CONTRIBUTORS.txt
 
-"""
-Various helper utilities.
-"""
+"""Various helper utilities."""
 
 from __future__ import annotations
 
-from astroid import bases, manager, nodes, raw_building, util
+import warnings
+from collections.abc import Generator
+
+from astroid import bases, manager, nodes, objects, raw_building, util
 from astroid.context import CallContext, InferenceContext
 from astroid.exceptions import (
     AstroidTypeError,
@@ -19,6 +20,20 @@ from astroid.exceptions import (
 )
 from astroid.nodes import scoped_nodes
 from astroid.typing import InferenceResult
+from astroid.util import safe_infer as real_safe_infer
+
+
+def safe_infer(
+    node: nodes.NodeNG | bases.Proxy | util.UninferableBase,
+    context: InferenceContext | None = None,
+) -> InferenceResult | None:
+    # When removing, also remove the real_safe_infer alias
+    warnings.warn(
+        "Import safe_infer from astroid.util; this shim in astroid.helpers will be removed.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return real_safe_infer(node, context=context)
 
 
 def _build_proxy_class(cls_name: str, builtins: nodes.Module) -> nodes.ClassDef:
@@ -28,9 +43,10 @@ def _build_proxy_class(cls_name: str, builtins: nodes.Module) -> nodes.ClassDef:
 
 
 def _function_type(
-    function: nodes.Lambda | bases.UnboundMethod, builtins: nodes.Module
+    function: nodes.Lambda | nodes.FunctionDef | bases.UnboundMethod,
+    builtins: nodes.Module,
 ) -> nodes.ClassDef:
-    if isinstance(function, scoped_nodes.Lambda):
+    if isinstance(function, (scoped_nodes.Lambda, scoped_nodes.FunctionDef)):
         if function.root().name == "builtins":
             cls_name = "builtin_function_or_method"
         else:
@@ -42,7 +58,9 @@ def _function_type(
     return _build_proxy_class(cls_name, builtins)
 
 
-def _object_type(node, context=None):
+def _object_type(
+    node: InferenceResult, context: InferenceContext | None = None
+) -> Generator[InferenceResult | None]:
     astroid_manager = manager.AstroidManager()
     builtins = astroid_manager.builtins_module
     context = context or InferenceContext()
@@ -55,18 +73,27 @@ def _object_type(node, context=None):
                     yield metaclass
                     continue
             yield builtins.getattr("type")[0]
-        elif isinstance(inferred, (scoped_nodes.Lambda, bases.UnboundMethod)):
+        elif isinstance(
+            inferred,
+            (scoped_nodes.Lambda, bases.UnboundMethod, scoped_nodes.FunctionDef),
+        ):
             yield _function_type(inferred, builtins)
         elif isinstance(inferred, scoped_nodes.Module):
             yield _build_proxy_class("module", builtins)
         elif isinstance(inferred, nodes.Unknown):
             raise InferenceError
-        else:
+        elif isinstance(inferred, util.UninferableBase):
+            yield inferred
+        elif isinstance(inferred, (bases.Proxy, nodes.Slice, objects.Super)):
             yield inferred._proxied
+        else:  # pragma: no cover
+            raise AssertionError(f"We don't handle {type(inferred)} currently")
 
 
-def object_type(node, context=None):
-    """Obtain the type of the given node
+def object_type(
+    node: InferenceResult, context: InferenceContext | None = None
+) -> InferenceResult | None:
+    """Obtain the type of the given node.
 
     This is used to implement the ``type`` builtin, which means that it's
     used for inferring type calls, as well as used in a couple of other places
@@ -81,28 +108,29 @@ def object_type(node, context=None):
         return util.Uninferable
     if len(types) > 1 or not types:
         return util.Uninferable
-    return list(types)[0]
+    return next(iter(types))
 
 
-def _object_type_is_subclass(obj_type, class_or_seq, context=None):
-    if not isinstance(class_or_seq, (tuple, list)):
-        class_seq = (class_or_seq,)
-    else:
-        class_seq = class_or_seq
-
-    if obj_type is util.Uninferable:
+def _object_type_is_subclass(
+    obj_type: InferenceResult | None,
+    class_or_seq: list[InferenceResult],
+    context: InferenceContext | None = None,
+) -> util.UninferableBase | bool:
+    if isinstance(obj_type, util.UninferableBase) or not isinstance(
+        obj_type, nodes.ClassDef
+    ):
         return util.Uninferable
 
     # Instances are not types
     class_seq = [
         item if not isinstance(item, bases.Instance) else util.Uninferable
-        for item in class_seq
+        for item in class_or_seq
     ]
     # strict compatibility with issubclass
     # issubclass(type, (object, 1)) evaluates to true
     # issubclass(object, (1, type)) raises TypeError
     for klass in class_seq:
-        if klass is util.Uninferable:
+        if isinstance(klass, util.UninferableBase):
             raise AstroidTypeError("arg 2 must be a type or tuple of types")
 
         for obj_subclass in obj_type.mro():
@@ -111,27 +139,27 @@ def _object_type_is_subclass(obj_type, class_or_seq, context=None):
     return False
 
 
-def object_isinstance(node, class_or_seq, context=None):
-    """Check if a node 'isinstance' any node in class_or_seq
-
-    :param node: A given node
-    :param class_or_seq: Union[nodes.NodeNG, Sequence[nodes.NodeNG]]
-    :rtype: bool
+def object_isinstance(
+    node: InferenceResult,
+    class_or_seq: list[InferenceResult],
+    context: InferenceContext | None = None,
+) -> util.UninferableBase | bool:
+    """Check if a node 'isinstance' any node in class_or_seq.
 
     :raises AstroidTypeError: if the given ``classes_or_seq`` are not types
     """
     obj_type = object_type(node, context)
-    if obj_type is util.Uninferable:
+    if isinstance(obj_type, util.UninferableBase):
         return util.Uninferable
     return _object_type_is_subclass(obj_type, class_or_seq, context=context)
 
 
-def object_issubclass(node, class_or_seq, context=None):
-    """Check if a type is a subclass of any node in class_or_seq
-
-    :param node: A given node
-    :param class_or_seq: Union[Nodes.NodeNG, Sequence[nodes.NodeNG]]
-    :rtype: bool
+def object_issubclass(
+    node: nodes.NodeNG,
+    class_or_seq: list[InferenceResult],
+    context: InferenceContext | None = None,
+) -> util.UninferableBase | bool:
+    """Check if a type is a subclass of any node in class_or_seq.
 
     :raises AstroidTypeError: if the given ``classes_or_seq`` are not types
     :raises AstroidError: if the type of the given node cannot be inferred
@@ -142,36 +170,14 @@ def object_issubclass(node, class_or_seq, context=None):
     return _object_type_is_subclass(node, class_or_seq, context=context)
 
 
-def safe_infer(
-    node: nodes.NodeNG | bases.Proxy, context: InferenceContext | None = None
-) -> InferenceResult | None:
-    """Return the inferred value for the given node.
-
-    Return None if inference failed or if there is some ambiguity (more than
-    one node has been inferred).
-    """
-    try:
-        inferit = node.infer(context=context)
-        value = next(inferit)
-    except (InferenceError, StopIteration):
-        return None
-    try:
-        next(inferit)
-        return None  # None if there is ambiguity on the inferred node
-    except InferenceError:
-        return None  # there is some kind of ambiguity
-    except StopIteration:
-        return value
-
-
-def has_known_bases(klass, context=None):
-    """Return true if all base classes of a class could be inferred."""
+def has_known_bases(klass, context: InferenceContext | None = None) -> bool:
+    """Return whether all base classes of a class could be inferred."""
     try:
         return klass._all_bases_known
     except AttributeError:
         pass
     for base in klass.bases:
-        result = safe_infer(base, context=context)
+        result = real_safe_infer(base, context=context)
         # TODO: check for A->B->A->B pattern in class structure too?
         if (
             not isinstance(result, scoped_nodes.ClassDef)
@@ -184,7 +190,7 @@ def has_known_bases(klass, context=None):
     return True
 
 
-def _type_check(type1, type2):
+def _type_check(type1, type2) -> bool:
     if not all(map(has_known_bases, (type1, type2))):
         raise _NonDeducibleTypeHierarchy
 
@@ -197,17 +203,17 @@ def _type_check(type1, type2):
         raise _NonDeducibleTypeHierarchy from e
 
 
-def is_subtype(type1, type2):
+def is_subtype(type1, type2) -> bool:
     """Check if *type1* is a subtype of *type2*."""
     return _type_check(type1=type2, type2=type1)
 
 
-def is_supertype(type1, type2):
+def is_supertype(type1, type2) -> bool:
     """Check if *type2* is a supertype of *type1*."""
     return _type_check(type1, type2)
 
 
-def class_instance_as_index(node):
+def class_instance_as_index(node: bases.Instance) -> nodes.Const | None:
     """Get the value as an index for the given instance.
 
     If an instance provides an __index__ method, then it can
@@ -230,8 +236,8 @@ def class_instance_as_index(node):
     return None
 
 
-def object_len(node, context=None):
-    """Infer length of given node object
+def object_len(node, context: InferenceContext | None = None):
+    """Infer length of given node object.
 
     :param Union[nodes.ClassDef, nodes.Instance] node:
     :param node: Node to infer length of
@@ -246,15 +252,15 @@ def object_len(node, context=None):
     # pylint: disable=import-outside-toplevel; circular import
     from astroid.objects import FrozenSet
 
-    inferred_node = safe_infer(node, context=context)
+    inferred_node = real_safe_infer(node, context=context)
 
     # prevent self referential length calls from causing a recursion error
-    # see https://github.com/PyCQA/astroid/issues/777
-    node_frame = node.frame(future=True)
+    # see https://github.com/pylint-dev/astroid/issues/777
+    node_frame = node.frame()
     if (
         isinstance(node_frame, scoped_nodes.FunctionDef)
         and node_frame.name == "__len__"
-        and hasattr(inferred_node, "_proxied")
+        and isinstance(inferred_node, bases.Proxy)
         and inferred_node._proxied == node_frame.parent
     ):
         message = (
@@ -265,7 +271,7 @@ def object_len(node, context=None):
         )
         raise InferenceError(message)
 
-    if inferred_node is None or inferred_node is util.Uninferable:
+    if inferred_node is None or isinstance(inferred_node, util.UninferableBase):
         raise InferenceError(node=node)
     if isinstance(inferred_node, nodes.Const) and isinstance(
         inferred_node.value, (bytes, str)
@@ -290,7 +296,7 @@ def object_len(node, context=None):
         ) from e
 
     inferred = len_call.infer_call_result(node, context)
-    if inferred is util.Uninferable:
+    if isinstance(inferred, util.UninferableBase):
         raise InferenceError(node=node, context=context)
     result_of_len = next(inferred, None)
     if (
@@ -298,9 +304,8 @@ def object_len(node, context=None):
         and result_of_len.pytype() == "builtins.int"
     ):
         return result_of_len.value
-    if (
-        result_of_len is None
-        or isinstance(result_of_len, bases.Instance)
+    if result_of_len is None or (
+        isinstance(result_of_len, bases.Instance)
         and result_of_len.is_subtype_of("builtins.int")
     ):
         # Fake a result as we don't know the arguments of the instance call.
@@ -308,3 +313,25 @@ def object_len(node, context=None):
     raise AstroidTypeError(
         f"'{result_of_len}' object cannot be interpreted as an integer"
     )
+
+
+def _higher_function_scope(node: nodes.NodeNG) -> nodes.FunctionDef | None:
+    """Search for the first function which encloses the given
+    scope.
+
+    This can be used for looking up in that function's
+    scope, in case looking up in a lower scope for a particular
+    name fails.
+
+    :param node: A scope node.
+    :returns:
+        ``None``, if no parent function scope was found,
+        otherwise an instance of :class:`astroid.nodes.scoped_nodes.Function`,
+        which encloses the given node.
+    """
+    current = node
+    while current.parent and not isinstance(current.parent, nodes.FunctionDef):
+        current = current.parent
+    if current and current.parent:
+        return current.parent
+    return None
